@@ -13,6 +13,7 @@ from mud.util import null_space
 from mud.funs import mud_sol, map_sol
 from mud.norm import full_functional, norm_input, norm_data, norm_predicted
 from mud.plot import make_2d_unit_mesh
+from mud_examples.helpers import check_dir
 
 
 def fit_log_linear_regression(input_values, output_values):
@@ -27,8 +28,10 @@ def plot_2d_contour_example(A=np.array([[1, 1]]), b=np.zeros([1, 1]),  # noqa: C
                             cov_11=0.5, cov_01=-0.25,
                             initial_mean=np.array([0.25, 0.25]),
                             alpha=1, omega=1, obs_std=1,
-                            show_full=True, show_data=True, show_est=False,
-                            fsize=42, figname='latest_figure.png', save=False):
+                            show_full=True, show_data=True,
+                            show_est=False, param_ref=None, compare=False,
+                            fsize=42, figname='latest_figure.png', save=False,
+                            ):
     """
     alpha: float in [0, 1], weight of Tikhonov regularization
     omega: float in [0, 1], weight of Modified regularization
@@ -94,6 +97,7 @@ def plot_2d_contour_example(A=np.array([[1, 1]]), b=np.zeros([1, 1]),  # noqa: C
                     cmap=cm.viridis, alpha=0.25)
     plt.axis('equal')
 
+
     if alpha + omega > 0:
         plt.scatter(initial_mean[0], initial_mean[1],
                     label='Initial Mean',
@@ -102,6 +106,17 @@ def plot_2d_contour_example(A=np.array([[1, 1]]), b=np.zeros([1, 1]),  # noqa: C
             plt.annotate('Initial Mean',
                          (initial_mean[0] + 0.001 * fsize, initial_mean[1] - 0.001 * fsize),
                          fontsize=fsize, backgroundcolor="w")
+        else:
+            if param_ref is not None:
+                plt.scatter(param_ref[0], param_ref[1],
+                    label='$\\lambda^\dagger$',
+                    color='k', s=msize, marker='*')
+            if compare:
+                plt.annotate('Truth',
+                         (param_ref[0] + 0.0005 * fsize, param_ref[1] + 0.0005 * fsize),
+                         fontsize=fsize, backgroundcolor="w")
+
+        show_mud = omega > 0 or compare
 
         if show_full:
             # scatter and line from origin to least squares
@@ -125,6 +140,7 @@ def plot_2d_contour_example(A=np.array([[1, 1]]), b=np.zeros([1, 1]),  # noqa: C
                     plt.scatter(map_pt[0], map_pt[1],
                                 label='min: Tk', color='xkcd:blue',
                                 marker='o', s=3 * msize, zorder=10)
+
             if (alpha > 0 and omega != 1):  # analytical MAP point
                 map_pt_eq = map_sol(A, b, observed_data_mean,
                                     initial_mean, initial_cov,
@@ -132,10 +148,18 @@ def plot_2d_contour_example(A=np.array([[1, 1]]), b=np.zeros([1, 1]),  # noqa: C
                 plt.scatter(map_pt_eq[0], map_pt_eq[1],
                             label='MAP', color='xkcd:orange',
                             marker='x', s=msize, lw=10, zorder=10)
-                plt.annotate('MAP',
-                             (map_pt_eq[0] + 0.001 * fsize, map_pt_eq[1] - 0.001 * fsize),
-                             fontsize=fsize, backgroundcolor="w")
-            if omega > 0:  # analytical MUD point
+
+                if compare:  # second map point has half the regularization strength
+                    plt.annotate('MAP$_{\\alpha}$',
+                                 (map_pt_eq[0] - 0.004 * fsize, map_pt_eq[1] - 0.002 * fsize),
+                                 fontsize=fsize, backgroundcolor="w")
+
+                else:
+                    plt.annotate('MAP$_{\\alpha}$',
+                                 (map_pt_eq[0] + 0.0001 * fsize, map_pt_eq[1] - 0.002 * fsize),
+                                 fontsize=fsize, backgroundcolor="w")
+
+            if show_mud:  # analytical MUD point
                 mud_pt_eq = mud_sol(A, b, observed_data_mean,
                                     initial_mean, initial_cov)
                 plt.scatter(mud_pt_eq[0], mud_pt_eq[1],
@@ -151,7 +175,7 @@ def plot_2d_contour_example(A=np.array([[1, 1]]), b=np.zeros([1, 1]),  # noqa: C
             v = v[::-1]  # in 2D, we can just swap entries and put a negative sign in front of one
             v[0] = - v[0]
 
-            if show_full and omega > 0:
+            if show_full and show_mud:
                 # grid search to find upper/lower bounds of line being drawn.
                 # importance is the direction, image is nicer with a proper origin/termination
                 s = np.linspace(-1, 1, 1000)
@@ -173,6 +197,9 @@ def plot_2d_contour_example(A=np.array([[1, 1]]), b=np.zeros([1, 1]),  # noqa: C
     plt.yticks(fontsize=0.75 * fsize)
     plt.tight_layout()
     if save:
+        if '/' in figname:
+            fdir = ''.join(figname.split('/')[:-1])
+            check_dir(fdir)
         plt.savefig(figname, dpi=300)
 
 #     plt.title('Predicted Covariance: {}'.format((A@initial_cov@A.T).ravel() ))
